@@ -1,30 +1,5 @@
-/* libmpdclient
-   (c) 2003-2019 The Music Player Daemon Project
-   This project's homepage is: http://www.musicpd.org
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright
-   notice, this list of conditions and the following disclaimer.
-
-   - Redistributions in binary form must reproduce the above copyright
-   notice, this list of conditions and the following disclaimer in the
-   documentation and/or other materials provided with the distribution.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// SPDX-License-Identifier: BSD-2-Clause
+// Copyright The Music Player Daemon Project
 
 #include <mpd/queue.h>
 #include <mpd/send.h>
@@ -56,7 +31,7 @@ mpd_send_list_queue_range_meta(struct mpd_connection *connection,
 bool
 mpd_send_get_queue_song_pos(struct mpd_connection *connection, unsigned pos)
 {
-	return mpd_send_int_command(connection, "playlistinfo", pos);
+	return mpd_send_u_command(connection, "playlistinfo", pos);
 }
 
 struct mpd_song *
@@ -81,7 +56,7 @@ mpd_run_get_queue_song_pos(struct mpd_connection *connection, unsigned pos)
 bool
 mpd_send_get_queue_song_id(struct mpd_connection *connection, unsigned id)
 {
-	return mpd_send_int_command(connection, "playlistid", id);
+	return mpd_send_u_command(connection, "playlistid", id);
 }
 
 struct mpd_song *
@@ -107,7 +82,7 @@ bool
 mpd_send_queue_changes_meta(struct mpd_connection *connection,
 			    unsigned version)
 {
-	return mpd_send_ll_command(connection, "plchanges", version);
+	return mpd_send_u_command(connection, "plchanges", version);
 }
 
 bool
@@ -123,7 +98,7 @@ bool
 mpd_send_queue_changes_brief(struct mpd_connection *connection,
 			     unsigned version)
 {
-	return mpd_send_ll_command(connection, "plchangesposid", version);
+	return mpd_send_u_command(connection, "plchangesposid", version);
 }
 
 bool
@@ -145,7 +120,7 @@ mpd_recv_queue_change_brief(struct mpd_connection *connection,
 	if (pair == NULL)
 		return false;
 
-	*position_r = atoi(pair->value);
+	*position_r = strtoul(pair->value, NULL, 10);
 	mpd_return_pair(connection, pair);
 
 	pair = mpd_recv_pair_named(connection, "Id");
@@ -162,16 +137,16 @@ mpd_recv_queue_change_brief(struct mpd_connection *connection,
 		return false;
 	}
 
-	*id_r = atoi(pair->value);
+	*id_r = strtoul(pair->value, NULL, 10);
 	mpd_return_pair(connection, pair);
 
 	return !mpd_error_is_defined(&connection->error);
 }
 
 bool
-mpd_send_add(struct mpd_connection *connection, const char *file)
+mpd_send_add(struct mpd_connection *connection, const char *uri)
 {
-	return mpd_send_command(connection, "add", file, NULL);
+	return mpd_send_command(connection, "add", uri, NULL);
 }
 
 bool
@@ -183,9 +158,30 @@ mpd_run_add(struct mpd_connection *connection, const char *uri)
 }
 
 bool
-mpd_send_add_id(struct mpd_connection *connection, const char *file)
+mpd_send_add_whence(struct mpd_connection *connection, const char *uri,
+		   unsigned to, enum mpd_position_whence whence)
 {
-	return mpd_send_command(connection, "addid", file, NULL);
+	const char *whence_s = mpd_position_whence_char(whence);
+
+	char to_str[64] = "";
+	snprintf(to_str, 64, "%s%u", whence_s, to);
+
+	return mpd_send_s_s_command(connection, "add", uri, to_str);
+}
+
+bool
+mpd_run_add_whence(struct mpd_connection *connection, const char *uri,
+		  unsigned to, enum mpd_position_whence whence)
+{
+	return mpd_run_check(connection) &&
+		mpd_send_add_whence(connection, uri, to, whence) &&
+		mpd_response_finish(connection);
+}
+
+bool
+mpd_send_add_id(struct mpd_connection *connection, const char *uri)
+{
+	return mpd_send_command(connection, "addid", uri, NULL);
 }
 
 bool
@@ -195,6 +191,18 @@ mpd_send_add_id_to(struct mpd_connection *connection, const char *uri,
 	return mpd_send_s_u_command(connection, "addid", uri, to);
 }
 
+bool
+mpd_send_add_id_whence(struct mpd_connection *connection, const char *uri,
+		   unsigned to, enum mpd_position_whence whence)
+{
+	const char *whence_s = mpd_position_whence_char(whence);
+
+	char to_str[64] = "";
+	snprintf(to_str, 64, "%s%u", whence_s, to);
+
+	return mpd_send_s_s_command(connection, "addid", uri, to_str);
+}
+
 int
 mpd_recv_song_id(struct mpd_connection *connection)
 {
@@ -211,12 +219,12 @@ mpd_recv_song_id(struct mpd_connection *connection)
 }
 
 int
-mpd_run_add_id(struct mpd_connection *connection, const char *file)
+mpd_run_add_id(struct mpd_connection *connection, const char *uri)
 {
 	int id;
 
 	if (!mpd_run_check(connection) ||
-	    !mpd_send_add_id(connection, file))
+	    !mpd_send_add_id(connection, uri))
 		return -1;
 
 	id = mpd_recv_song_id(connection);
@@ -245,10 +253,28 @@ mpd_run_add_id_to(struct mpd_connection *connection, const char *uri,
 	return id;
 }
 
+int
+mpd_run_add_id_whence(struct mpd_connection *connection, const char *uri,
+		  unsigned to, enum mpd_position_whence whence)
+{
+	int id;
+
+	if (!mpd_run_check(connection) ||
+	    !mpd_send_add_id_whence(connection, uri, to, whence))
+		return -1;
+
+	id = mpd_recv_song_id(connection);
+
+	if (!mpd_response_finish(connection))
+		id = -1;
+
+	return id;
+}
+
 bool
 mpd_send_delete(struct mpd_connection *connection, unsigned pos)
 {
-	return mpd_send_int_command(connection, "delete", pos);
+	return mpd_send_u_command(connection, "delete", pos);
 }
 
 bool
@@ -278,7 +304,7 @@ mpd_run_delete_range(struct mpd_connection *connection,
 bool
 mpd_send_delete_id(struct mpd_connection *connection, unsigned id)
 {
-	return mpd_send_int_command(connection, "deleteid", id);
+	return mpd_send_u_command(connection, "deleteid", id);
 }
 
 bool
@@ -335,7 +361,7 @@ mpd_run_clear(struct mpd_connection *connection)
 bool
 mpd_send_move(struct mpd_connection *connection, unsigned from, unsigned to)
 {
-	return mpd_send_int2_command(connection, "move", from, to);
+	return mpd_send_u2_command(connection, "move", from, to);
 }
 
 bool
@@ -346,10 +372,31 @@ mpd_run_move(struct mpd_connection *connection, unsigned from, unsigned to)
 		mpd_response_finish(connection);
 }
 
+bool
+mpd_send_move_whence(struct mpd_connection *connection, unsigned from,
+			 unsigned to, enum mpd_position_whence whence)
+{
+	const char *whence_s = mpd_position_whence_char(whence);
+
+	char to_str[64] = "";
+	snprintf(to_str, 64, "%s%u", whence_s, to);
+
+	return mpd_send_u_s_command(connection, "move", from, to_str);
+}
+
+bool
+mpd_run_move_whence(struct mpd_connection *connection, unsigned from,
+			 unsigned to, enum mpd_position_whence whence)
+{
+	return mpd_run_check(connection) &&
+		mpd_send_move_whence(connection, from, to, whence) &&
+		mpd_response_finish(connection);
+}
+
 bool
 mpd_send_move_id(struct mpd_connection *connection, unsigned from, unsigned to)
 {
-	return mpd_send_int2_command(connection, "moveid", from, to);
+	return mpd_send_u2_command(connection, "moveid", from, to);
 }
 
 bool
@@ -360,6 +407,27 @@ mpd_run_move_id(struct mpd_connection *connection, unsigned from, unsigned to)
 		mpd_response_finish(connection);
 }
 
+bool
+mpd_send_move_id_whence(struct mpd_connection *connection, unsigned from,
+			 unsigned to, enum mpd_position_whence whence)
+{
+	const char *whence_s = mpd_position_whence_char(whence);
+
+	char to_str[64] = "";
+	snprintf(to_str, 64, "%s%u", whence_s, to);
+
+	return mpd_send_u_s_command(connection, "moveid", from, to_str);
+}
+
+bool
+mpd_run_move_id_whence(struct mpd_connection *connection, unsigned from,
+			 unsigned to, enum mpd_position_whence whence)
+{
+	return mpd_run_check(connection) &&
+		mpd_send_move_id_whence(connection, from, to, whence) &&
+		mpd_response_finish(connection);
+}
+
 bool
 mpd_send_move_range(struct mpd_connection *connection,
 		    unsigned start, unsigned end, unsigned to)
@@ -376,10 +444,31 @@ mpd_run_move_range(struct mpd_connection *connection,
 		mpd_response_finish(connection);
 }
 
+bool
+mpd_send_move_range_whence(struct mpd_connection *connection, unsigned start,
+			 unsigned end, unsigned to, enum mpd_position_whence whence)
+{
+	const char *whence_s = mpd_position_whence_char(whence);
+
+	char to_str[64] = "";
+	snprintf(to_str, 64, "%s%u", whence_s, to);
+
+	return mpd_send_range_to_command(connection, "move", start, end, to_str);
+}
+
+bool
+mpd_run_move_range_whence(struct mpd_connection *connection, unsigned start,
+			 unsigned end, unsigned to, enum mpd_position_whence whence)
+{
+	return mpd_run_check(connection) &&
+		mpd_send_move_range_whence(connection, start, end, to, whence) &&
+		mpd_response_finish(connection);
+}
+
 bool
 mpd_send_swap(struct mpd_connection *connection, unsigned pos1, unsigned pos2)
 {
-	return mpd_send_int2_command(connection, "swap", pos1, pos2);
+	return mpd_send_u2_command(connection, "swap", pos1, pos2);
 }
 
 bool
@@ -393,7 +482,7 @@ mpd_run_swap(struct mpd_connection *connection, unsigned pos1, unsigned pos2)
 bool
 mpd_send_swap_id(struct mpd_connection *connection, unsigned id1, unsigned id2)
 {
-	return mpd_send_int2_command(connection, "swapid", id1, id2);
+	return mpd_send_u2_command(connection, "swapid", id1, id2);
 }
 
 bool
@@ -441,7 +530,7 @@ mpd_run_clear_tag_id(struct mpd_connection *connection, unsigned id,
 bool
 mpd_send_clear_all_tags_id(struct mpd_connection *connection, unsigned id)
 {
-	return mpd_send_int_command(connection, "cleartagid", id);
+	return mpd_send_u_command(connection, "cleartagid", id);
 }
 
 bool
@@ -453,14 +542,14 @@ mpd_run_clear_all_tags_id(struct mpd_connection *connection, unsigned id)
 }
 
 bool
-mpd_send_prio(struct mpd_connection *connection, int priority,
+mpd_send_prio(struct mpd_connection *connection, unsigned priority,
 	      unsigned position)
 {
-	return mpd_send_int2_command(connection, "prio", priority, position);
+	return mpd_send_u2_command(connection, "prio", priority, position);
 }
 
 bool
-mpd_run_prio(struct mpd_connection *connection, int priority,
+mpd_run_prio(struct mpd_connection *connection, unsigned priority,
 	     unsigned position)
 {
 	return mpd_run_check(connection) &&
@@ -469,15 +558,15 @@ mpd_run_prio(struct mpd_connection *connection, int priority,
 }
 
 bool
-mpd_send_prio_range(struct mpd_connection *connection, int priority,
+mpd_send_prio_range(struct mpd_connection *connection, unsigned priority,
 		    unsigned start, unsigned end)
 {
-	return mpd_send_i_range_command(connection, "prio", priority,
+	return mpd_send_u_range_command(connection, "prio", priority,
 					start, end);
 }
 
 bool
-mpd_run_prio_range(struct mpd_connection *connection, int priority,
+mpd_run_prio_range(struct mpd_connection *connection, unsigned priority,
 		   unsigned start, unsigned end)
 {
 	return mpd_run_check(connection) &&
@@ -486,14 +575,14 @@ mpd_run_prio_range(struct mpd_connection *connection, int priority,
 }
 
 bool
-mpd_send_prio_id(struct mpd_connection *connection, int priority,
+mpd_send_prio_id(struct mpd_connection *connection, unsigned priority,
 		 unsigned id)
 {
-	return mpd_send_int2_command(connection, "prioid", priority, id);
+	return mpd_send_u2_command(connection, "prioid", priority, id);
 }
 
 bool
-mpd_run_prio_id(struct mpd_connection *connection, int priority,
+mpd_run_prio_id(struct mpd_connection *connection, unsigned priority,
 		unsigned id)
 {
 	return mpd_run_check(connection) &&
